@@ -6,6 +6,7 @@ from collections import defaultdict, deque
 from collections.abc import Container
 from math import log2
 from time import time
+from typing import TYPE_CHECKING, ClassVar, TypedDict
 
 from tlz import topk
 from tornado.ioloop import PeriodicCallback
@@ -18,6 +19,10 @@ from distributed.core import CommClosedError, Status
 from distributed.diagnostics.plugin import SchedulerPlugin
 from distributed.utils import log_errors, recursive_to_dict
 
+if TYPE_CHECKING:
+    # Recursive imports
+    from distributed.scheduler import Scheduler, TaskState, WorkerState
+
 # Stealing requires multiple network bounces and if successful also task
 # submission which may include code serialization. Therefore, be very
 # conservative in the latency estimation to suppress too aggressive stealing
@@ -48,18 +53,40 @@ _WORKER_STATE_UNDEFINED = {
 }
 
 
-class WorkStealing(SchedulerPlugin):
-    def __init__(self, scheduler):
-        self.scheduler = scheduler
-        # { level: { task states } }
-        self.stealable_all = [set() for i in range(15)]
-        # { worker: { level: { task states } } }
-        self.stealable = dict()
-        # { task state: (worker, level) }
-        self.key_stealable = dict()
+class InFlightInfo(TypedDict):
+    victim: WorkerState
+    thief: WorkerState
+    victim_duration: float
+    thief_duration: float
+    stimulus_id: str
 
-        self.cost_multipliers = [1 + 2 ** (i - 6) for i in range(15)]
-        self.cost_multipliers[0] = 1
+
+class WorkStealing(SchedulerPlugin):
+    scheduler: Scheduler
+    # ({ task states for level 0}, ..., {task states for level 14})
+    stealable_all: tuple[set[TaskState], ...]
+    # {worker: ({ task states for level 0}, ..., {task states for level 14})}
+    stealable: dict[str, tuple[set[TaskState], ...]]
+    # { task state: (worker, level) }
+    key_stealable: dict[TaskState, tuple[str, int]]
+    # (multiplier for level 0, ... multiplier for level 14)
+    cost_multipliers: ClassVar[tuple[float, ...]] = (1.0,) + tuple(
+        1 + 2 ** (i - 6) for i in range(1, 15)
+    )
+    _callback_time: float | None
+    count: int
+    # { task state: <stealing info dict> }
+    in_flight: dict[TaskState, InFlightInfo]
+    # { worker state: occupancy }
+    in_flight_occupancy: defaultdict[WorkerState, float]
+    _in_flight_event: asyncio.Event
+    _request_counter: int
+
+    def __init__(self, scheduler: Scheduler):
+        self.scheduler = scheduler
+        self.stealable_all = tuple(set() for _ in range(15))
+        self.stealable = {}
+        self.key_stealable = {}
 
         for worker in scheduler.workers:
             self.add_worker(worker=worker)
@@ -72,9 +99,7 @@ class WorkStealing(SchedulerPlugin):
         self.scheduler.add_plugin(self)
         self.scheduler.events["stealing"] = deque(maxlen=100000)
         self.count = 0
-        # { task state: <stealing info dict> }
-        self.in_flight = dict()
-        # { worker state: occupancy }
+        self.in_flight = {}
         self.in_flight_occupancy = defaultdict(lambda: 0)
         self._in_flight_event = asyncio.Event()
         self._request_counter = 0
@@ -121,9 +146,9 @@ class WorkStealing(SchedulerPlugin):
         return self.scheduler.log_event("stealing", msg)
 
     def add_worker(self, scheduler=None, worker=None):
-        self.stealable[worker] = [set() for i in range(15)]
+        self.stealable[worker] = tuple(set() for _ in range(15))
 
-    def remove_worker(self, scheduler=None, worker=None):
+    def remove_worker(self, scheduler: Scheduler, worker: str) -> None:
         del self.stealable[worker]
 
     def teardown(self):
@@ -213,7 +238,9 @@ class WorkStealing(SchedulerPlugin):
 
         return cost_multiplier, level
 
-    def move_task_request(self, ts, victim, thief) -> str:
+    def move_task_request(
+        self, ts: TaskState, victim: WorkerState, thief: WorkerState
+    ) -> str:
         try:
             if ts in self.in_flight:
                 return "in-flight"
@@ -283,7 +310,6 @@ class WorkStealing(SchedulerPlugin):
 
         thief = d["thief"]
         victim = d["victim"]
-
         logger.debug("Confirm move %s, %s -> %s.  State: %s", key, victim, thief, state)
 
         self.in_flight_occupancy[thief] -= d["thief_duration"]
@@ -304,8 +330,9 @@ class WorkStealing(SchedulerPlugin):
                 self.scheduler._reevaluate_occupancy_worker(victim)
             elif (
                 state in _WORKER_STATE_UNDEFINED
+                # If our steal information is somehow stale we need to reschedule
                 or state in _WORKER_STATE_CONFIRM
-                and thief.address not in self.scheduler.workers
+                and thief != self.scheduler.workers.get(thief.address)
             ):
                 self.log(
                     (
@@ -351,17 +378,17 @@ class WorkStealing(SchedulerPlugin):
     def balance(self):
         s = self.scheduler
 
-        def combined_occupancy(ws):
+        def combined_occupancy(ws: WorkerState) -> float:
             return ws.occupancy + self.in_flight_occupancy[ws]
 
         def maybe_move_task(
-            level,
-            ts,
-            victim,
-            thief,
+            level: int,
+            ts: TaskState,
+            victim: WorkerState,
+            thief: WorkerState,
             duration: float,
             cost_multiplier: float,
-        ):
+        ) -> None:
             occ_thief = combined_occupancy(thief)
             occ_victim = combined_occupancy(victim)
 
